@@ -1,8 +1,8 @@
 /*
  * sample_buffer.h - container-class sampleBuffer
  *
- * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
- * 
+ * Copyright (c) 2005-2009 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
@@ -26,33 +26,15 @@
 #ifndef _SAMPLE_BUFFER_H
 #define _SAMPLE_BUFFER_H
 
-#include "qt3support.h"
-
-#ifdef QT4
-
+#include <QtCore/QMutex>
 #include <QtCore/QObject>
+#include <QtCore/QRect>
 
-#else
-
-#include <qobject.h>
-
-#endif
-
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifndef USE_3RDPARTY_LIBSRC
 #include <samplerate.h>
-#else
-#include "src/3rdparty/samplerate/samplerate.h"
-#endif
 
-
-#include "mixer.h"
+#include "export.h"
 #include "interpolation.h"
-#include "types.h"
+#include "lmms_basics.h"
 #include "lmms_math.h"
 #include "shared_object.h"
 
@@ -60,28 +42,21 @@
 class QPainter;
 
 
-class sampleBuffer : public QObject, public sharedObject
+class EXPORT sampleBuffer : public QObject, public sharedObject
 {
 	Q_OBJECT
 public:
-	enum drawMethods
-	{
-		LINE_CONNECT,
-		DOTS
-	} ;
-
-
-	class handleState
+	class EXPORT handleState
 	{
 	public:
-		handleState( bool _varying_pitch = FALSE );
+		handleState( bool _varying_pitch = false );
 		virtual ~handleState();
 
 
 	private:
-		f_cnt_t m_frame_index;
-		const bool m_varying_pitch;
-		SRC_STATE * m_resampling_data;
+		f_cnt_t m_frameIndex;
+		const bool m_varyingPitch;
+		SRC_STATE * m_resamplingData;
 
 		friend class sampleBuffer;
 
@@ -90,108 +65,122 @@ public:
 
 	// constructor which either loads sample _audio_file or decodes
 	// base64-data out of string
-	sampleBuffer( const QString & _audio_file = "",
-						bool _is_base64_data = FALSE );
+	sampleBuffer( const QString & _audio_file = QString(),
+						bool _is_base64_data = false );
 	sampleBuffer( const sampleFrame * _data, const f_cnt_t _frames );
 	sampleBuffer( const f_cnt_t _frames );
-	
+
 	virtual ~sampleBuffer();
 
-	bool FASTCALL play( sampleFrame * _ab, handleState * _state,
+	bool play( sampleFrame * _ab, handleState * _state,
 				const fpp_t _frames,
-				const float _freq = BASE_FREQ,
-				const bool _looped = FALSE );
+				const float _freq,
+				const bool _looped = false );
 
-	void FASTCALL visualize( QPainter & _p, const QRect & _dr,
-					const QRect & _clip,
-					drawMethods _dm = LINE_CONNECT );
-	inline void visualize( QPainter & _p, const QRect & _dr,
-					drawMethods _dm = LINE_CONNECT )
+	void visualize( QPainter & _p, const QRect & _dr, const QRect & _clip, f_cnt_t _from_frame = 0, f_cnt_t _to_frame = 0 );
+	inline void visualize( QPainter & _p, const QRect & _dr, f_cnt_t _from_frame = 0, f_cnt_t _to_frame = 0 )
 	{
-		visualize( _p, _dr, _dr, _dm );
+		visualize( _p, _dr, _dr, _from_frame, _to_frame );
 	}
 
-	inline const QString & audioFile( void ) const
+	inline const QString & audioFile() const
 	{
-		return( m_audioFile );
+		return m_audioFile;
 	}
 
-	inline f_cnt_t startFrame( void ) const
+	inline f_cnt_t startFrame() const
 	{
-		return( m_startFrame );
+		return m_startFrame;
 	}
 
-	inline f_cnt_t endFrame( void ) const
+	inline f_cnt_t endFrame() const
 	{
-		return( m_endFrame );
+		return m_endFrame;
 	}
 
 	void setLoopStartFrame( f_cnt_t _start )
 	{
-		m_loop_startFrame = _start;
+		m_varLock.lock();
+		m_loopStartFrame = _start;
+		m_varLock.unlock();
 	}
 
 	void setLoopEndFrame( f_cnt_t _end )
 	{
-		m_loop_endFrame = _end;
+		m_varLock.lock();
+		m_loopEndFrame = _end;
+		m_varLock.unlock();
 	}
 
-	inline f_cnt_t frames( void ) const
+	inline f_cnt_t frames() const
 	{
-		return( m_frames );
+		return m_frames;
 	}
 
-	inline float amplification( void ) const
+	inline float amplification() const
 	{
-		return( m_amplification );
+		return m_amplification;
 	}
 
-	inline bool reversed( void ) const
+	inline bool reversed() const
 	{
-		return( m_reversed );
+		return m_reversed;
 	}
 
-	inline float frequency( void ) const
+	inline float frequency() const
 	{
-		return( m_frequency );
+		return m_frequency;
+	}
+
+	sample_rate_t sampleRate() const
+	{
+		return m_sampleRate;
+	}
+
+	int sampleLength() const
+	{
+		return double( m_endFrame - m_startFrame ) / m_sampleRate * 1000;
 	}
 
 	inline void setFrequency( float _freq )
 	{
+		m_varLock.lock();
 		m_frequency = _freq;
+		m_varLock.unlock();
 	}
 
 	inline void setSampleRate( sample_rate_t _rate )
 	{
-		m_sample_rate = _rate;
+		m_varLock.lock();
+		m_sampleRate = _rate;
+		m_varLock.unlock();
 	}
 
-	inline const sampleFrame * data( void ) const
+	inline const sampleFrame * data() const
 	{
-		return( m_data );
+		return m_data;
 	}
 
-	QString openAudioFile( void ) const;
+	QString openAudioFile() const;
 
 	QString & toBase64( QString & _dst ) const;
 
 
-	static sampleBuffer * FASTCALL resample( sampleFrame * _data,
+	static sampleBuffer * resample( sampleFrame * _data,
 						const f_cnt_t _frames,
 						const sample_rate_t _src_sr,
 						const sample_rate_t _dst_sr );
 
-	static inline sampleBuffer * FASTCALL resample(
-						sampleBuffer * _buf,
+	static inline sampleBuffer * resample( sampleBuffer * _buf,
 						const sample_rate_t _src_sr,
 						const sample_rate_t _dst_sr )
 	{
-		return( resample( _buf->m_data, _buf->m_frames, _src_sr,
-								_dst_sr ) );
+		return resample( _buf->m_data, _buf->m_frames, _src_sr,
+								_dst_sr );
 	}
 
-	void normalize_sample_rate( const sample_rate_t _src_sr,
-						bool _keep_settings = FALSE );
+	void normalizeSampleRate( const sample_rate_t _src_sr,
+						bool _keep_settings = false );
 
 	inline sample_t userWaveSample( const float _sample ) const
 	{
@@ -202,7 +191,7 @@ public:
 //		sample_t waveSample = linearInterpolate( m_data[f1][0],
 //						m_data[f2][0],
 //						fraction( frame ) );
-//		return( waveSample );
+//		return waveSample;
 
 		// Fast implementation
 		const float frame = _sample * m_frames;
@@ -211,7 +200,7 @@ public:
 		{
 			f1 += m_frames;
 		}
-		return( m_data[f1][0] );
+		return m_data[f1][0];
 	}
 
 	static QString tryToMakeRelative( const QString & _file );
@@ -228,59 +217,44 @@ public slots:
 
 
 private:
-	void FASTCALL update( bool _keep_settings = FALSE );
+	void update( bool _keep_settings = false );
 
 
-#ifdef SDL_SDL_SOUND_H
-	f_cnt_t FASTCALL decodeSampleSDL( const char * _f,
-						int_sample_t * & _buf,
-						ch_cnt_t _channels,
-						sample_rate_t _sample_rate );
-#endif
-#ifdef HAVE_SNDFILE_H
-	f_cnt_t FASTCALL decodeSampleSF( const char * _f,
-						int_sample_t * & _buf,
+	f_cnt_t decodeSampleSF( const char * _f, int_sample_t * & _buf,
+						ch_cnt_t & _channels,
+						sample_rate_t & _sample_rate );
+#ifdef LMMS_HAVE_OGGVORBIS
+	f_cnt_t decodeSampleOGGVorbis( const char * _f, int_sample_t * & _buf,
 						ch_cnt_t & _channels,
 						sample_rate_t & _sample_rate );
 #endif
-#ifdef HAVE_VORBIS_VORBISFILE_H
-	f_cnt_t FASTCALL decodeSampleOGGVorbis( const char * _f,
-						int_sample_t * & _buf,
+	f_cnt_t decodeSampleDS( const char * _f, int_sample_t * & _buf,
 						ch_cnt_t & _channels,
 						sample_rate_t & _sample_rate );
-#endif
-	f_cnt_t FASTCALL decodeSampleDS( const char * _f,
-						int_sample_t * & _buf,
-						ch_cnt_t & _channels,
-						sample_rate_t & _sample_rate );
-
 
 	QString m_audioFile;
 	sampleFrame * m_origData;
 	f_cnt_t m_origFrames;
 	sampleFrame * m_data;
+	QMutex m_varLock;
 	f_cnt_t m_frames;
 	f_cnt_t m_startFrame;
 	f_cnt_t m_endFrame;
-	f_cnt_t m_loop_startFrame;
-	f_cnt_t m_loop_endFrame;
+	f_cnt_t m_loopStartFrame;
+	f_cnt_t m_loopEndFrame;
 	float m_amplification;
 	bool m_reversed;
 	float m_frequency;
-	sample_rate_t m_sample_rate;
+	sample_rate_t m_sampleRate;
 
-	void initResampling( void );
-
-	SRC_DATA m_srcData;
-
-	sampleFrame * m_sample_fragment;
 	sampleFrame * getSampleFragment( f_cnt_t _start, f_cnt_t _frames,
-								bool _looped );
-	f_cnt_t getLoopedIndex( f_cnt_t _index );
+						bool _looped,
+						sampleFrame * * _tmp ) const;
+	f_cnt_t getLoopedIndex( f_cnt_t _index ) const;
 
 
 signals:
-	void sampleUpdated( void );
+	void sampleUpdated();
 
 } ;
 
