@@ -1,7 +1,7 @@
 /*
  * song.h - class song - the root of the model-tree
  *
- * Copyright (c) 2004-2011 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2012 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -25,13 +25,14 @@
 #ifndef _SONG_H
 #define _SONG_H
 
+#include <QtCore/QSharedMemory>
 #include <QtCore/QVector>
 
 #include "track_container.h"
 #include "AutomatableModel.h"
 #include "Controller.h"
 #include "MeterModel.h"
-
+#include "VST_sync_shm.h"
 
 class AutomationTrack;
 class pattern;
@@ -111,6 +112,11 @@ public:
 		return m_exporting;
 	}
 
+	inline void setExportLoop( bool exportLoop )
+	{
+		m_exportLoop = exportLoop;
+	}
+
 	inline bool isRecording() const
 	{
 		return m_recording;
@@ -120,8 +126,16 @@ public:
 
 	inline bool isExportDone() const
 	{
-		return m_exporting == true &&
-			m_playPos[Mode_PlaySong].getTact() >= length() + 1;
+		if ( m_exportLoop )
+		{
+			return m_exporting == true &&
+				m_playPos[Mode_PlaySong].getTact() >= length();
+		}
+		else
+		{
+			return m_exporting == true &&
+				m_playPos[Mode_PlaySong].getTact() >= length() + 1;
+		}
 	}
 
 	inline PlayModes playMode() const
@@ -207,7 +221,8 @@ public slots:
 	void resumeFromPause();
 
 	void importProject();
-	void exportProject();
+	void exportProject(bool multiExport=false);
+	void exportProjectTracks();
 
 	void startExport();
 	void stopExport();
@@ -235,6 +250,8 @@ private slots:
 
 	void updateFramesPerTick();
 
+	void updateSampleRateSHM();
+
 
 
 private:
@@ -282,6 +299,7 @@ private:
 
 	volatile bool m_recording;
 	volatile bool m_exporting;
+	volatile bool m_exportLoop;
 	volatile bool m_playing;
 	volatile bool m_paused;
 
@@ -308,6 +326,9 @@ private:
 	} ;
 	QVector<Actions> m_actions;
 
+	int m_shmID;
+	sncVST * m_SncVSTplug;
+	QSharedMemory m_shmQtID;
 
 	friend class engine;
 	friend class songEditor;
