@@ -2,7 +2,7 @@
  * sf2_player.cpp - a soundfont2 player using fluidSynth
  *
  * Copyright (c) 2008 Paul Giblock <drfaygo/at/gmail/dot/com>
- * Copyright (c) 2009 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2009-2013 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -32,13 +32,12 @@
 
 #include "sf2_player.h"
 #include "engine.h"
-#include "instrument_track.h"
-#include "instrument_play_handle.h"
+#include "InstrumentTrack.h"
+#include "InstrumentPlayHandle.h"
 #include "note_play_handle.h"
 #include "knob.h"
 #include "song.h"
 
-#include "main_window.h"
 #include "patches_dialog.h"
 #include "tooltip.h"
 #include "lcd_spinbox.h"
@@ -49,7 +48,7 @@
 extern "C"
 {
 
-plugin::descriptor sf2player_plugin_descriptor =
+Plugin::Descriptor PLUGIN_EXPORT sf2player_plugin_descriptor =
 {
 	STRINGIFY( PLUGIN_NAME ),
 	"Sf2 Player",
@@ -57,8 +56,8 @@ plugin::descriptor sf2player_plugin_descriptor =
 			"Player for SoundFont files" ),
 	"Paul Giblock <drfaygo/at/gmail/dot/com>",
 	0x0100,
-	plugin::Instrument,
-	new pluginPixmapLoader( "logo" ),
+	Plugin::Instrument,
+	new PluginPixmapLoader( "logo" ),
 	"sf2",
 	NULL
 } ;
@@ -82,8 +81,8 @@ QMutex sf2Instrument::s_fontsMutex;
 
 
 
-sf2Instrument::sf2Instrument( instrumentTrack * _instrument_track ) :
-	instrument( _instrument_track, &sf2player_plugin_descriptor ),
+sf2Instrument::sf2Instrument( InstrumentTrack * _instrument_track ) :
+	Instrument( _instrument_track, &sf2player_plugin_descriptor ),
 	m_srcState( NULL ),
 	m_font( NULL ),
 	m_fontId( 0 ),
@@ -93,7 +92,7 @@ sf2Instrument::sf2Instrument( instrumentTrack * _instrument_track ) :
 	m_bankNum( 0, 0, 999, this, tr("Bank") ),
 	m_patchNum( 0, 0, 127, this, tr("Patch") ),
 	m_gain( 1.0f, 0.0f, 5.0f, 0.01f, this, tr( "Gain" ) ),
-	m_reverbOn( FALSE, this, tr( "Reverb" ) ),
+	m_reverbOn( false, this, tr( "Reverb" ) ),
 	m_reverbRoomSize( FLUID_REVERB_DEFAULT_ROOMSIZE, 0, 1.0, 0.01f,
 			this, tr( "Reverb Roomsize" ) ),
 	m_reverbDamping( FLUID_REVERB_DEFAULT_DAMP, 0, 1.0, 0.01,
@@ -102,7 +101,7 @@ sf2Instrument::sf2Instrument( instrumentTrack * _instrument_track ) :
 			this, tr( "Reverb Width" ) ),
 	m_reverbLevel( FLUID_REVERB_DEFAULT_LEVEL, 0, 1.0, 0.01f,
 			this, tr( "Reverb Level" ) ),
-	m_chorusOn( FALSE, this, tr( "Chorus" ) ),
+	m_chorusOn( false, this, tr( "Chorus" ) ),
 	m_chorusNum( FLUID_CHORUS_DEFAULT_N, 0, 10.0, 1.0,
 			this, tr( "Chorus Lines" ) ),
 	m_chorusLevel( FLUID_CHORUS_DEFAULT_LEVEL, 0, 10.0, 0.01,
@@ -126,10 +125,10 @@ sf2Instrument::sf2Instrument( instrumentTrack * _instrument_track ) :
 	// everytime we load a new soundfont.
 	m_synth = new_fluid_synth( m_settings );
 
-	instrumentPlayHandle * iph = new instrumentPlayHandle( this );
+	InstrumentPlayHandle * iph = new InstrumentPlayHandle( this );
 	engine::getMixer()->addPlayHandle( iph );
 
-	//loadFile( configManager::inst()->defaultSoundfont() );
+	loadFile( configManager::inst()->defaultSoundfont() );
 
 	updateSampleRate();
 	updateReverbOn();
@@ -190,7 +189,7 @@ sf2Instrument::sf2Instrument( instrumentTrack * _instrument_track ) :
 
 sf2Instrument::~sf2Instrument()
 {
-	engine::getMixer()->removePlayHandles( getInstrumentTrack() );
+	engine::getMixer()->removePlayHandles( instrumentTrack() );
 	freeFont();
 	delete_fluid_synth( m_synth );
 	delete_fluid_settings( m_settings );
@@ -229,7 +228,7 @@ void sf2Instrument::saveSettings( QDomDocument & _doc, QDomElement & _this )
 
 void sf2Instrument::loadSettings( const QDomElement & _this )
 {
-	openFile( _this.attribute( "src" ) );
+	openFile( _this.attribute( "src" ), false );
 	m_patchNum.loadSettings( _this, "patch" );
 	m_bankNum.loadSettings( _this, "bank" );
 
@@ -256,7 +255,7 @@ void sf2Instrument::loadSettings( const QDomElement & _this )
 
 void sf2Instrument::loadFile( const QString & _file )
 {
-	if( !_file.isEmpty() )
+	if( !_file.isEmpty() && QFileInfo( _file ).exists() )
 	{
 		openFile( _file );
 		updatePatch();
@@ -270,7 +269,7 @@ void sf2Instrument::loadFile( const QString & _file )
 
 
 
-automatableModel * sf2Instrument::getChildModel( const QString & _modelName )
+AutomatableModel * sf2Instrument::childModel( const QString & _modelName )
 {
 	if( _modelName == "bank" )
 	{
@@ -310,7 +309,7 @@ void sf2Instrument::freeFont()
 		{
 			cout << "Really deleting " << m_filename << endl;
 
-			fluid_synth_sfunload( m_synth, m_fontId, TRUE );
+			fluid_synth_sfunload( m_synth, m_fontId, true );
 			s_fonts.remove( m_filename );
 			delete m_font;
 		}
@@ -330,7 +329,7 @@ void sf2Instrument::freeFont()
 
 
 
-void sf2Instrument::openFile( const QString & _sf2File )
+void sf2Instrument::openFile( const QString & _sf2File, bool updateTrackName )
 {
 	emit fileLoading();
 
@@ -361,7 +360,7 @@ void sf2Instrument::openFile( const QString & _sf2File )
 	// Add to map, if doesn't exist.
 	else
 	{
-		m_fontId = fluid_synth_sfload( m_synth, sf2Ascii, TRUE );
+		m_fontId = fluid_synth_sfload( m_synth, sf2Ascii, true );
 
 		if( fluid_synth_sfcount( m_synth ) > 0 )
 		{
@@ -390,6 +389,11 @@ void sf2Instrument::openFile( const QString & _sf2File )
 	}
 
 	delete[] sf2Ascii;
+
+	if( updateTrackName )
+	{
+		instrumentTrack()->setName( QFileInfo( _sf2File ).baseName() );
+	}
 }
 
 
@@ -609,7 +613,7 @@ void sf2Instrument::playNote( notePlayHandle * _n, sampleFrame * )
 		}
 
 		fluid_synth_noteon( m_synth, m_channel, midiNote,
-							_n->getMidiVelocity() );
+							_n->midiVelocity() );
 
 		// get new voice and save it
 		fluid_synth_get_voicelist( m_synth, voices, poly, -1 );
@@ -632,6 +636,7 @@ void sf2Instrument::playNote( notePlayHandle * _n, sampleFrame * )
 
 	SF2PluginData * pluginData = static_cast<SF2PluginData *>(
 							_n->m_pluginData );
+#ifdef SOMEONE_FIXED_PER_NOTE_PANNING
 	if( pluginData->fluidVoice &&
 			pluginData->lastPanning != _n->getPanning() )
 	{
@@ -646,6 +651,7 @@ void sf2Instrument::playNote( notePlayHandle * _n, sampleFrame * )
 
 		pluginData->lastPanning = _n->getPanning();
 	}
+#endif
 
 	const float currentVelocity = _n->volumeLevel( tfp ) * 127;
 	if( pluginData->fluidVoice &&
@@ -674,9 +680,9 @@ void sf2Instrument::play( sampleFrame * _working_buffer )
 	const fpp_t frames = engine::getMixer()->framesPerPeriod();
 
 	m_synthMutex.lock();
-	if( m_lastMidiPitch != getInstrumentTrack()->midiPitch() )
+	if( m_lastMidiPitch != instrumentTrack()->midiPitch() )
 	{
-		m_lastMidiPitch = getInstrumentTrack()->midiPitch();
+		m_lastMidiPitch = instrumentTrack()->midiPitch();
 		fluid_synth_pitch_bend( m_synth, m_channel, m_lastMidiPitch );
 	}
 
@@ -721,7 +727,7 @@ void sf2Instrument::play( sampleFrame * _working_buffer )
 	}
 	m_synthMutex.unlock();
 
-	getInstrumentTrack()->processAudioBuffer( _working_buffer, frames,
+	instrumentTrack()->processAudioBuffer( _working_buffer, frames,
 									NULL );
 }
 
@@ -749,7 +755,7 @@ void sf2Instrument::deleteNotePluginData( notePlayHandle * _n )
 
 
 
-pluginView * sf2Instrument::instantiateView( QWidget * _parent )
+PluginView * sf2Instrument::instantiateView( QWidget * _parent )
 {
 	return new sf2InstrumentView( this, _parent );
 }
@@ -772,9 +778,9 @@ public:
 
 
 
-sf2InstrumentView::sf2InstrumentView( instrument * _instrument,
+sf2InstrumentView::sf2InstrumentView( Instrument * _instrument,
 			QWidget * _parent ) :
-	instrumentView( _instrument, _parent )
+	InstrumentView( _instrument, _parent )
 {
 //	QVBoxLayout * vl = new QVBoxLayout( this );
 //	QHBoxLayout * hl = new QHBoxLayout();
@@ -808,7 +814,7 @@ sf2InstrumentView::sf2InstrumentView( instrument * _instrument,
 							"patches_on" ) );
 	m_patchDialogButton->setInactiveGraphic( PLUGIN_NAME::getIconPixmap(
 							"patches_off" ) );
-	m_patchDialogButton->setEnabled( FALSE );
+	m_patchDialogButton->setEnabled( false );
 	m_patchDialogButton->move( 217, 125 );
 
 	connect( m_patchDialogButton, SIGNAL( clicked() ),
@@ -820,12 +826,12 @@ sf2InstrumentView::sf2InstrumentView( instrument * _instrument,
 	m_bankNumLcd = new lcdSpinBox( 3, "21pink", this );
 	m_bankNumLcd->move(131, 62);
 //	m_bankNumLcd->addTextForValue( -1, "---" );
-//	m_bankNumLcd->setEnabled( FALSE );
+//	m_bankNumLcd->setEnabled( false );
 
 	m_patchNumLcd = new lcdSpinBox( 3, "21pink", this );
 	m_patchNumLcd->move(190, 62);
 //	m_patchNumLcd->addTextForValue( -1, "---" );
-//	m_patchNumLcd->setEnabled( FALSE );
+//	m_patchNumLcd->setEnabled( false );
 
 	/*hl->addWidget( m_fileDialogButton );
 	hl->addWidget( m_bankNumLcd );
@@ -857,7 +863,7 @@ sf2InstrumentView::sf2InstrumentView( instrument * _instrument,
 
 	
 	m_reverbButton = new pixmapButton( this );
-	m_reverbButton->setCheckable( TRUE );
+	m_reverbButton->setCheckable( true );
 	m_reverbButton->move( 24, 176 );
 	m_reverbButton->setActiveGraphic( PLUGIN_NAME::getIconPixmap(
 							"reverb_on" ) );
@@ -899,7 +905,7 @@ sf2InstrumentView::sf2InstrumentView( instrument * _instrument,
 //	hl = new QHBoxLayout();
 
 	m_chorusButton = new pixmapButton( this );
-	m_chorusButton->setCheckable( TRUE );
+	m_chorusButton->setCheckable( true );
 	m_chorusButton->move( 24, 222 );
 	m_chorusButton->setActiveGraphic( PLUGIN_NAME::getIconPixmap(
 							"chorus_on" ) );
@@ -935,7 +941,7 @@ sf2InstrumentView::sf2InstrumentView( instrument * _instrument,
 
 	vl->addLayout( hl );
 */
-	setAutoFillBackground( TRUE );
+	setAutoFillBackground( true );
 	QPalette pal;
 	pal.setBrush( backgroundRole(), PLUGIN_NAME::getIconPixmap(
 								"artwork" ) );
@@ -1053,7 +1059,7 @@ void sf2InstrumentView::showFileDialog()
 		if( QFileInfo( f ).isRelative() )
 		{
 			f = configManager::inst()->userSamplesDir() + f;
-			if( QFileInfo( f ).exists() == FALSE )
+			if( QFileInfo( f ).exists() == false )
 			{
 				f = configManager::inst()->factorySamplesDir() +
 						k->m_filename;
@@ -1091,7 +1097,7 @@ void sf2InstrumentView::showPatchDialog()
 
 	patchesDialog pd( this );
 
-	pd.setup( k->m_synth, 1, k->getInstrumentTrack()->name(),
+	pd.setup( k->m_synth, 1, k->instrumentTrack()->name(),
 		&k->m_bankNum, &k->m_patchNum, m_patchLabel );
 
 	pd.exec();
@@ -1102,11 +1108,10 @@ void sf2InstrumentView::showPatchDialog()
 extern "C"
 {
 
-// neccessary for getting instance out of shared lib
-plugin * lmms_plugin_main( model *, void * _data )
+// necessary for getting instance out of shared lib
+Plugin * PLUGIN_EXPORT lmms_plugin_main( Model *, void * _data )
 {
-	return( new sf2Instrument(
-			static_cast<instrumentTrack *>( _data ) ) );
+	return new sf2Instrument( static_cast<InstrumentTrack *>( _data ) );
 }
 
 
