@@ -3,6 +3,7 @@
  *                      of ladspa plugins
  *
  * Copyright (c) 2005-2008 Danny McRae <khjklujn@netscape.net>
+ * Copyright (c) 2011 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -23,7 +24,7 @@
  *
  */
 
-
+#include <QtCore/QDebug>
 #include <QtCore/QDir>
 #include <QtCore/QFileInfo>
 #include <QtCore/QLibrary>
@@ -35,10 +36,10 @@
 
 
 
-ladspaManager::ladspaManager( void )
+ladspaManager::ladspaManager()
 {
 	QStringList ladspaDirectories = QString( getenv( "LADSPA_PATH" ) ).
-								split( ',' );
+								split( LADSPA_PATH_SEPERATOR );
 	ladspaDirectories += configManager::inst()->ladspaDir().split( ',' );
 
 	ladspaDirectories.push_back( configManager::inst()->pluginDir() + "ladspa" );
@@ -60,10 +61,11 @@ ladspaManager::ladspaManager( void )
 		{
 			const QFileInfo & f = *file;
 			if( !f.isFile() ||
+				 f.fileName().right( 3 ).toLower() !=
 #ifdef LMMS_BUILD_WIN32
-				 f.fileName().right( 3 ).toLower() != "dll"
+													"dll"
 #else
-				 f.fileName().right( 2 ).toLower() != "so"
+				 									".so"
 #endif
 								)
 			{
@@ -72,7 +74,7 @@ ladspaManager::ladspaManager( void )
 
 			QLibrary plugin_lib( f.absoluteFilePath() );
 
-			if( plugin_lib.load() == TRUE )
+			if( plugin_lib.load() == true )
 			{
 				LADSPA_Descriptor_Function descriptorFunction =
 			( LADSPA_Descriptor_Function ) plugin_lib.resolve(
@@ -83,6 +85,10 @@ ladspaManager::ladspaManager( void )
 							f.fileName() );
 				}
 			}
+			else
+			{
+				qWarning() << plugin_lib.errorString();
+			}
 		}
 	}
 	
@@ -269,7 +275,7 @@ bool ladspaManager::hasRealTimeDependency(
 	}
 	else
 	{
-		return( FALSE );
+		return( false );
 	}
 }
 
@@ -289,7 +295,7 @@ bool ladspaManager::isInplaceBroken( const ladspa_key_t &  _plugin )
 	}
 	else
 	{
-		return( FALSE );
+		return( false );
 	}
 }
 
@@ -310,7 +316,7 @@ bool ladspaManager::isRealTimeCapable(
 	}
 	else
 	{
-		return( FALSE );
+		return( false );
 	}
 }
 
@@ -414,7 +420,7 @@ bool ladspaManager::isPortInput( const ladspa_key_t & _plugin,
 	}
 	else
 	{
-		return( FALSE );
+		return( false );
 	}
 }
 
@@ -438,7 +444,7 @@ bool ladspaManager::isPortOutput( const ladspa_key_t & _plugin,
 	}
 	else
 	{
-		return( FALSE );
+		return( false );
 	}
 }
 
@@ -462,7 +468,7 @@ bool ladspaManager::isPortAudio( const ladspa_key_t & _plugin,
 	}
 	else
 	{
-		return( FALSE );
+		return( false );
 	}
 }
 
@@ -486,7 +492,7 @@ bool ladspaManager::isPortControl( const ladspa_key_t & _plugin,
 	}
 	else
 	{
-		return( FALSE );
+		return( false );
 	}
 }
 
@@ -511,7 +517,7 @@ bool ladspaManager::areHintsSampleRateDependent(
 	}
 	else
 	{
-		return( FALSE );
+		return( false );
 	}
 }
 
@@ -596,7 +602,7 @@ bool ladspaManager::isPortToggled( const ladspa_key_t & _plugin,
 	}
 	else
 	{
-		return( FALSE );
+		return( false );
 	}
 }
 
@@ -707,7 +713,7 @@ bool ladspaManager::isLogarithmic( const ladspa_key_t & _plugin,
 	}
 	else
 	{
-		return( FALSE );
+		return( false );
 	}
 }
 
@@ -731,7 +737,7 @@ bool ladspaManager::isInteger( const ladspa_key_t & _plugin,
 	}
 	else
 	{
-		return( FALSE );
+		return( false );
 	}
 }
 
@@ -843,10 +849,10 @@ bool ladspaManager::connectPort( const ladspa_key_t & _plugin,
 		{
 			( descriptor->connect_port )
 					( _instance, _port, _data_location );
-			return( TRUE );
+			return( true );
 		}
 	}
-	return( FALSE );
+	return( false );
 }
 
 
@@ -865,10 +871,10 @@ bool ladspaManager::activate( const ladspa_key_t & _plugin,
 		if( descriptor->activate != NULL )
 		{
 			( descriptor->activate ) ( _instance );
-			return( TRUE );
+			return( true );
 		}
 	}
-	return( FALSE );
+	return( false );
 }
 
 
@@ -888,10 +894,10 @@ bool ladspaManager::run( const ladspa_key_t & _plugin,
 		if( descriptor->run != NULL )
 		{
 			( descriptor->run ) ( _instance, _sample_count );
-			return( TRUE );
+			return( true );
 		}
 	}
-	return( FALSE );
+	return( false );
 }
 
 
@@ -912,10 +918,10 @@ bool ladspaManager::runAdding( const ladspa_key_t & _plugin,
 			  	descriptor->set_run_adding_gain != NULL )
 		{
 			( descriptor->run_adding ) ( _instance, _sample_count );
-			return( TRUE );
+			return( true );
 		}
 	}
-	return( FALSE );
+	return( false );
 }
 
 
@@ -937,10 +943,10 @@ bool ladspaManager::setRunAddingGain( const ladspa_key_t & _plugin,
 		{
 			( descriptor->set_run_adding_gain )
 							( _instance, _gain );
-			return( TRUE );
+			return( true );
 		}
 	}
-	return( FALSE );
+	return( false );
 }
 
 
@@ -959,10 +965,10 @@ bool ladspaManager::deactivate( const ladspa_key_t & _plugin,
 		if( descriptor->deactivate != NULL )
 		{
 			( descriptor->deactivate ) ( _instance );
-			return( TRUE );
+			return( true );
 		}
 	}
-	return( FALSE );
+	return( false );
 }
 
 
@@ -981,8 +987,8 @@ bool ladspaManager::cleanup( const ladspa_key_t & _plugin,
 		if( descriptor->cleanup != NULL )
 		{
 			( descriptor->cleanup ) ( _instance );
-			return( TRUE );
+			return( true );
 		}
 	}
-	return( FALSE );
+	return( false );
 }
