@@ -1,10 +1,8 @@
-#ifndef SINGLE_SOURCE_COMPILE
-
 /*
  * setup_dialog.cpp - dialog for setting up LMMS
  *
- * Copyright (c) 2005-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
- * 
+ * Copyright (c) 2005-2013 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +22,6 @@
  *
  */
 
-
 #include <QtGui/QComboBox>
 #include <QtGui/QFileDialog>
 #include <QtGui/QImageReader>
@@ -35,14 +32,13 @@
 #include <QtGui/QSlider>
 #include <QtGui/QWhatsThis>
 
-
 #include "setup_dialog.h"
 #include "tab_bar.h"
 #include "tab_button.h"
 #include "tab_widget.h"
 #include "gui_templates.h"
 #include "mixer.h"
-#include "project_journal.h"
+#include "ProjectJournal.h"
 #include "config_mgr.h"
 #include "embed.h"
 #include "engine.h"
@@ -53,27 +49,28 @@
 
 
 // platform-specific audio-interface-classes
-#include "audio_alsa.h"
-#include "audio_jack.h"
-#include "audio_oss.h"
-#include "audio_portaudio.h"
-#include "audio_pulseaudio.h"
-#include "audio_sdl.h"
-#include "audio_dummy.h"
+#include "AudioAlsa.h"
+#include "AudioJack.h"
+#include "AudioOss.h"
+#include "AudioPortAudio.h"
+#include "AudioPulseAudio.h"
+#include "AudioSdl.h"
+#include "AudioDummy.h"
 
 // platform-specific midi-interface-classes
-#include "midi_alsa_raw.h"
-#include "midi_alsa_seq.h"
-#include "midi_oss.h"
-#include "midi_winmm.h"
-#include "midi_dummy.h"
+#include "MidiAlsaRaw.h"
+#include "MidiAlsaSeq.h"
+#include "MidiOss.h"
+#include "MidiWinMM.h"
+#include "MidiDummy.h"
+
 
 
 inline void labelWidget( QWidget * _w, const QString & _txt )
 {
 	QLabel * title = new QLabel( _txt, _w );
 	QFont f = title->font();
-	f.setBold( TRUE );
+	f.setBold( true );
 	title->setFont( pointSize<12>( f ) );
 
 #ifdef LMMS_DEBUG
@@ -114,13 +111,23 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 	m_disableChActInd( configManager::inst()->value( "ui",
 				"disablechannelactivityindicators" ).toInt() ),
 	m_manualChPiano( configManager::inst()->value( "ui",
-					"manualchannelpiano" ).toInt() )
+					"manualchannelpiano" ).toInt() ),
+	m_smoothScroll( configManager::inst()->value( "ui", "smoothscroll" ).toInt() ),
+	m_enableAutoSave( configManager::inst()->value( "ui", "enableautosave" ).toInt() ),
+	m_oneInstrumentTrackWindow( configManager::inst()->value( "ui",
+					"oneinstrumenttrackwindow" ).toInt() ),
+	m_compactTrackButtons( configManager::inst()->value( "ui",
+					"compacttrackbuttons" ).toInt() ),
+	m_syncVSTPlugins( configManager::inst()->value( "ui",
+							"syncvstplugins" ).toInt() ),
+	m_animateAFP(configManager::inst()->value( "ui",
+						   "animateafp").toInt() )
 {
 	setWindowIcon( embed::getIconPixmap( "setup_general" ) );
 	setWindowTitle( tr( "Setup LMMS" ) );
-	setModal( TRUE );
+	setModal( true );
 
-	engine::getProjectJournal()->setJournalling( FALSE );
+	engine::projectJournal()->setJournalling( false );
 
 	QVBoxLayout * vlayout = new QVBoxLayout( this );
 	vlayout->setSpacing( 0 );
@@ -131,7 +138,7 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 	hlayout->setMargin( 0 );
 
 	m_tabBar = new tabBar( settings, QBoxLayout::TopToBottom );
-	m_tabBar->setExclusive( TRUE );
+	m_tabBar->setExclusive( true );
 	m_tabBar->setFixedWidth( 72 );
 
 	QWidget * ws = new QWidget( settings );
@@ -183,7 +190,7 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 
 
 	tabWidget * misc_tw = new tabWidget( tr( "MISC" ), general );
-	misc_tw->setFixedHeight( 120 );
+	misc_tw->setFixedHeight( 174 );
 
 	ledCheckBox * enable_tooltips = new ledCheckBox(
 							tr( "Enable tooltips" ),
@@ -219,14 +226,39 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 	connect( mmpz, SIGNAL( toggled( bool ) ),
 					this, SLOT( toggleMMPZ( bool ) ) );
 
+	ledCheckBox * oneitw = new ledCheckBox(
+				tr( "One instrument track window mode" ),
+								misc_tw );
+	oneitw->move( 10, 90 );
+	oneitw->setChecked( m_oneInstrumentTrackWindow );
+	connect( oneitw, SIGNAL( toggled( bool ) ),
+				this, SLOT( toggleOneInstrumentTrackWindow( bool ) ) );
+
 	ledCheckBox * hqaudio = new ledCheckBox(
 				tr( "HQ-mode for output audio-device" ),
 								misc_tw );
-	hqaudio->move( 10, 90 );
+	hqaudio->move( 10, 108 );
 	hqaudio->setChecked( m_hqAudioDev );
 	connect( hqaudio, SIGNAL( toggled( bool ) ),
 				this, SLOT( toggleHQAudioDev( bool ) ) );
 
+	ledCheckBox * compacttracks = new ledCheckBox(
+				tr( "Compact track buttons" ),
+								misc_tw );
+	compacttracks->move( 10, 126 );
+	compacttracks->setChecked( m_compactTrackButtons );
+	connect( compacttracks, SIGNAL( toggled( bool ) ),
+				this, SLOT( toggleCompactTrackButtons( bool ) ) );
+
+
+	ledCheckBox * syncVST = new ledCheckBox(
+				tr( "Sync VST plugins to host playback" ),
+								misc_tw );
+	syncVST->move( 10, 144 );
+	syncVST->setChecked( m_syncVSTPlugins );
+	connect( syncVST, SIGNAL( toggled( bool ) ),
+				this, SLOT( toggleSyncVSTPlugins( bool ) ) );
+
 
 
 	gen_layout->addWidget( bufsize_tw );
@@ -446,7 +478,7 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 	tabWidget * ui_fx_tw = new tabWidget( tr( "UI effects vs. "
 						"performance" ).toUpper(),
 								performance );
-	ui_fx_tw->setFixedHeight( 70 );
+	ui_fx_tw->setFixedHeight( 120 );
 
 	ledCheckBox * disable_ch_act_ind = new ledCheckBox(
 				tr( "Disable channel activity indicators" ),
@@ -465,6 +497,30 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 	connect( manual_ch_piano, SIGNAL( toggled( bool ) ),
 				this, SLOT( toggleManualChPiano( bool ) ) );
 
+	ledCheckBox * smoothScroll = new ledCheckBox(
+			tr( "Smooth scroll in Song Editor" ), ui_fx_tw );
+	smoothScroll->move( 10, 60 );
+	smoothScroll->setChecked( m_smoothScroll );
+	connect( smoothScroll, SIGNAL( toggled( bool ) ),
+				this, SLOT( toggleSmoothScroll( bool ) ) );
+
+
+	ledCheckBox * autoSave = new ledCheckBox(
+			tr( "Enable auto save feature" ), ui_fx_tw );
+	autoSave->move( 10, 80 );
+	autoSave->setChecked( m_enableAutoSave );
+	connect( autoSave, SIGNAL( toggled( bool ) ),
+				this, SLOT( toggleAutoSave( bool ) ) );
+
+
+	ledCheckBox * animAFP = new ledCheckBox(
+				tr( "Show playback cursor in AudioFileProcessor" ),
+								ui_fx_tw );
+	animAFP->move( 10, 100 );
+	animAFP->setChecked( m_animateAFP );
+	connect( animAFP, SIGNAL( toggled( bool ) ),
+				this, SLOT( toggleAnimateAFP( bool ) ) );
+
 
 
 	perf_layout->addWidget( ui_fx_tw );
@@ -501,42 +557,42 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 	QHBoxLayout * asw_layout = new QHBoxLayout( asw );
 	asw_layout->setSpacing( 0 );
 	asw_layout->setMargin( 0 );
-	//asw_layout->setAutoAdd( TRUE );
+	//asw_layout->setAutoAdd( true );
 
 #ifdef LMMS_HAVE_JACK
-	m_audioIfaceSetupWidgets[audioJACK::name()] =
-					new audioJACK::setupWidget( asw );
+	m_audioIfaceSetupWidgets[AudioJack::name()] =
+					new AudioJack::setupWidget( asw );
 #endif
 
 #ifdef LMMS_HAVE_ALSA
-	m_audioIfaceSetupWidgets[audioALSA::name()] =
-					new audioALSA::setupWidget( asw );
+	m_audioIfaceSetupWidgets[AudioAlsa::name()] =
+					new AudioAlsa::setupWidget( asw );
 #endif
 
 #ifdef LMMS_HAVE_PULSEAUDIO
-	m_audioIfaceSetupWidgets[audioPulseAudio::name()] =
-					new audioPulseAudio::setupWidget( asw );
+	m_audioIfaceSetupWidgets[AudioPulseAudio::name()] =
+					new AudioPulseAudio::setupWidget( asw );
 #endif
 
 #ifdef LMMS_HAVE_PORTAUDIO
-	m_audioIfaceSetupWidgets[audioPortAudio::name()] =
-					new audioPortAudio::setupWidget( asw );
+	m_audioIfaceSetupWidgets[AudioPortAudio::name()] =
+					new AudioPortAudio::setupWidget( asw );
 #endif
 
 #ifdef LMMS_HAVE_SDL
-	m_audioIfaceSetupWidgets[audioSDL::name()] =
-					new audioSDL::setupWidget( asw );
+	m_audioIfaceSetupWidgets[AudioSdl::name()] =
+					new AudioSdl::setupWidget( asw );
 #endif
 
 #ifdef LMMS_HAVE_OSS
-	m_audioIfaceSetupWidgets[audioOSS::name()] =
-					new audioOSS::setupWidget( asw );
+	m_audioIfaceSetupWidgets[AudioOss::name()] =
+					new AudioOss::setupWidget( asw );
 #endif
-	m_audioIfaceSetupWidgets[audioDummy::name()] =
-					new audioDummy::setupWidget( asw );
+	m_audioIfaceSetupWidgets[AudioDummy::name()] =
+					new AudioDummy::setupWidget( asw );
 
 
-	for( aswMap::iterator it = m_audioIfaceSetupWidgets.begin();
+	for( AswMap::iterator it = m_audioIfaceSetupWidgets.begin();
 				it != m_audioIfaceSetupWidgets.end(); ++it )
 	{
 		m_audioIfaceNames[tr( it.key().toAscii())] = it.key();
@@ -593,30 +649,30 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 	QHBoxLayout * msw_layout = new QHBoxLayout( msw );
 	msw_layout->setSpacing( 0 );
 	msw_layout->setMargin( 0 );
-	//msw_layout->setAutoAdd( TRUE );
+	//msw_layout->setAutoAdd( true );
 
 #ifdef LMMS_HAVE_ALSA
-	m_midiIfaceSetupWidgets[midiALSASeq::name()] =
-					new midiALSASeq::setupWidget( msw );
-	m_midiIfaceSetupWidgets[midiALSARaw::name()] =
-					new midiALSARaw::setupWidget( msw );
+	m_midiIfaceSetupWidgets[MidiAlsaSeq::name()] =
+					new MidiAlsaSeq::setupWidget( msw );
+	m_midiIfaceSetupWidgets[MidiAlsaRaw::name()] =
+					new MidiAlsaRaw::setupWidget( msw );
 #endif
 
 #ifdef LMMS_HAVE_OSS
-	m_midiIfaceSetupWidgets[midiOSS::name()] =
-					new midiOSS::setupWidget( msw );
+	m_midiIfaceSetupWidgets[MidiOss::name()] =
+					new MidiOss::setupWidget( msw );
 #endif
 
 #ifdef LMMS_BUILD_WIN32
-	m_midiIfaceSetupWidgets[midiWinMM::name()] =
-					new midiWinMM::setupWidget( msw );
+	m_midiIfaceSetupWidgets[MidiWinMM::name()] =
+					new MidiWinMM::setupWidget( msw );
 #endif
 
-	m_midiIfaceSetupWidgets[midiDummy::name()] =
-					new midiDummy::setupWidget( msw );
+	m_midiIfaceSetupWidgets[MidiDummy::name()] =
+					new MidiDummy::setupWidget( msw );
 
 
-	for( mswMap::iterator it = m_midiIfaceSetupWidgets.begin();
+	for( MswMap::iterator it = m_midiIfaceSetupWidgets.begin();
 				it != m_midiIfaceSetupWidgets.end(); ++it )
 	{
 		m_midiIfaceNames[tr( it.key().toAscii())] = it.key();
@@ -644,17 +700,17 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 	midi_layout->addStretch();
 
 
-	m_tabBar->addTab( general, tr( "General settings" ), 0, FALSE, TRUE 
+	m_tabBar->addTab( general, tr( "General settings" ), 0, false, true 
 			)->setIcon( embed::getIconPixmap( "setup_general" ) );
-	m_tabBar->addTab( paths, tr( "Paths" ), 1, FALSE, TRUE 
+	m_tabBar->addTab( paths, tr( "Paths" ), 1, false, true 
 			)->setIcon( embed::getIconPixmap(
 							"setup_directories" ) );
-	m_tabBar->addTab( performance, tr( "Performance settings" ), 2, FALSE,
-				TRUE )->setIcon( embed::getIconPixmap(
+	m_tabBar->addTab( performance, tr( "Performance settings" ), 2, false,
+				true )->setIcon( embed::getIconPixmap(
 							"setup_performance" ) );
-	m_tabBar->addTab( audio, tr( "Audio settings" ), 3, FALSE, TRUE
+	m_tabBar->addTab( audio, tr( "Audio settings" ), 3, false, true
 			)->setIcon( embed::getIconPixmap( "setup_audio" ) );
-	m_tabBar->addTab( midi, tr( "MIDI settings" ), 4, TRUE, TRUE
+	m_tabBar->addTab( midi, tr( "MIDI settings" ), 4, true, true
 			)->setIcon( embed::getIconPixmap( "setup_midi" ) );
 
 
@@ -703,13 +759,13 @@ setupDialog::setupDialog( ConfigTabs _tab_to_open ) :
 
 setupDialog::~setupDialog()
 {
-	engine::getProjectJournal()->setJournalling( TRUE );
+	engine::projectJournal()->setJournalling( true );
 }
 
 
 
 
-void setupDialog::accept( void )
+void setupDialog::accept()
 {
 	configManager::inst()->setValue( "mixer", "framesperaudiobuffer",
 					QString::number( m_bufferSize ) );
@@ -732,6 +788,19 @@ void setupDialog::accept( void )
 					QString::number( m_disableChActInd ) );
 	configManager::inst()->setValue( "ui", "manualchannelpiano",
 					QString::number( m_manualChPiano ) );
+	configManager::inst()->setValue( "ui", "smoothscroll",
+					QString::number( m_smoothScroll ) );
+	configManager::inst()->setValue( "ui", "enableautosave",
+					QString::number( m_enableAutoSave ) );
+	configManager::inst()->setValue( "ui", "oneinstrumenttrackwindow",
+					QString::number( m_oneInstrumentTrackWindow ) );
+	configManager::inst()->setValue( "ui", "compacttrackbuttons",
+					QString::number( m_compactTrackButtons ) );
+	configManager::inst()->setValue( "ui", "syncvstplugins",
+					QString::number( m_syncVSTPlugins ) );
+	configManager::inst()->setValue( "ui", "animateafp",
+					QString::number( m_animateAFP ) );
+
 
 	configManager::inst()->setWorkingDir( m_workingDir );
 	configManager::inst()->setVSTDir( m_vstDir );
@@ -747,13 +816,13 @@ void setupDialog::accept( void )
 	configManager::inst()->setBackgroundArtwork( m_backgroundArtwork );
 
 	// tell all audio-settings-widget to save their settings
-	for( aswMap::iterator it = m_audioIfaceSetupWidgets.begin();
+	for( AswMap::iterator it = m_audioIfaceSetupWidgets.begin();
 				it != m_audioIfaceSetupWidgets.end(); ++it )
 	{
 		it.value()->saveSettings();
 	}
 	// tell all MIDI-settings-widget to save their settings
-	for( mswMap::iterator it = m_midiIfaceSetupWidgets.begin();
+	for( MswMap::iterator it = m_midiIfaceSetupWidgets.begin();
 				it != m_midiIfaceSetupWidgets.end(); ++it )
 	{
 		it.value()->saveSettings();
@@ -808,7 +877,7 @@ void setupDialog::setBufferSize( int _value )
 
 
 
-void setupDialog::resetBufSize( void )
+void setupDialog::resetBufSize()
 {
 	setBufferSize( DEFAULT_BUFFER_SIZE / 64 );
 }
@@ -816,7 +885,7 @@ void setupDialog::resetBufSize( void )
 
 
 
-void setupDialog::displayBufSizeHelp( void )
+void setupDialog::displayBufSizeHelp()
 {
 	QWhatsThis::showText( QCursor::pos(),
 			tr( "Here you can setup the internal buffer-size "
@@ -888,7 +957,56 @@ void setupDialog::toggleManualChPiano( bool _enabled )
 
 
 
-void setupDialog::openWorkingDir( void )
+void setupDialog::toggleSmoothScroll( bool _enabled )
+{
+	m_smoothScroll = _enabled;
+}
+
+
+
+void setupDialog::toggleAutoSave( bool _enabled )
+{
+	m_enableAutoSave = _enabled;
+}
+
+
+
+
+
+
+void setupDialog::toggleCompactTrackButtons( bool _enabled )
+{
+	m_compactTrackButtons = _enabled;
+}
+
+
+
+
+
+void setupDialog::toggleSyncVSTPlugins( bool _enabled )
+{
+	m_syncVSTPlugins = _enabled;
+}
+
+void setupDialog::toggleAnimateAFP( bool _enabled )
+{
+	m_animateAFP = _enabled;
+}
+
+
+
+
+
+void setupDialog::toggleOneInstrumentTrackWindow( bool _enabled )
+{
+	m_oneInstrumentTrackWindow = _enabled;
+}
+
+
+
+
+
+void setupDialog::openWorkingDir()
 {
 	QString new_dir = QFileDialog::getExistingDirectory( this,
 					tr( "Choose LMMS working directory" ),
@@ -910,7 +1028,7 @@ void setupDialog::setWorkingDir( const QString & _wd )
 
 
 
-void setupDialog::openVSTDir( void )
+void setupDialog::openVSTDir()
 {
 	QString new_dir = QFileDialog::getExistingDirectory( this,
 				tr( "Choose your VST-plugin directory" ),
@@ -932,7 +1050,7 @@ void setupDialog::setVSTDir( const QString & _vd )
 
 
 
-void setupDialog::openArtworkDir( void )
+void setupDialog::openArtworkDir()
 {
 	QString new_dir = QFileDialog::getExistingDirectory( this,
 				tr( "Choose artwork-theme directory" ),
@@ -954,7 +1072,7 @@ void setupDialog::setArtworkDir( const QString & _ad )
 
 
 
-void setupDialog::openFLDir( void )
+void setupDialog::openFLDir()
 {
 	QString new_dir = QFileDialog::getExistingDirectory( this,
 				tr( "Choose FL Studio installation directory" ),
@@ -968,7 +1086,7 @@ void setupDialog::openFLDir( void )
 
 
 
-void setupDialog::openLADSPADir( void )
+void setupDialog::openLADSPADir()
 {
 	QString new_dir = QFileDialog::getExistingDirectory( this,
 				tr( "Choose LADSPA plugin directory" ),
@@ -990,7 +1108,7 @@ void setupDialog::openLADSPADir( void )
 
 
 
-void setupDialog::openSTKDir( void )
+void setupDialog::openSTKDir()
 {
 #ifdef LMMS_HAVE_STK
 	QString new_dir = QFileDialog::getExistingDirectory( this,
@@ -1006,7 +1124,7 @@ void setupDialog::openSTKDir( void )
 
 
 
-void setupDialog::openDefaultSoundfont( void )
+void setupDialog::openDefaultSoundfont()
 {
 #ifdef LMMS_HAVE_FLUIDSYNTH
 	QString new_file = QFileDialog::getOpenFileName( this,
@@ -1023,7 +1141,7 @@ void setupDialog::openDefaultSoundfont( void )
 
 
 
-void setupDialog::openBackgroundArtwork( void )
+void setupDialog::openBackgroundArtwork()
 {
 	QList<QByteArray> fileTypesList = QImageReader::supportedImageFormats();
 	QString fileTypes;
@@ -1103,7 +1221,7 @@ void setupDialog::setBackgroundArtwork( const QString & _ba )
 
 void setupDialog::audioInterfaceChanged( const QString & _iface )
 {
-	for( aswMap::iterator it = m_audioIfaceSetupWidgets.begin();
+	for( AswMap::iterator it = m_audioIfaceSetupWidgets.begin();
 				it != m_audioIfaceSetupWidgets.end(); ++it )
 	{
 		it.value()->hide();
@@ -1115,7 +1233,7 @@ void setupDialog::audioInterfaceChanged( const QString & _iface )
 
 
 
-void setupDialog::displayAudioHelp( void )
+void setupDialog::displayAudioHelp()
 {
 	QWhatsThis::showText( QCursor::pos(),
 				tr( "Here you can select your preferred "
@@ -1133,7 +1251,7 @@ void setupDialog::displayAudioHelp( void )
 
 void setupDialog::midiInterfaceChanged( const QString & _iface )
 {
-	for( mswMap::iterator it = m_midiIfaceSetupWidgets.begin();
+	for( MswMap::iterator it = m_midiIfaceSetupWidgets.begin();
 				it != m_midiIfaceSetupWidgets.end(); ++it )
 	{
 		it.value()->hide();
@@ -1145,7 +1263,7 @@ void setupDialog::midiInterfaceChanged( const QString & _iface )
 
 
 
-void setupDialog::displayMIDIHelp( void )
+void setupDialog::displayMIDIHelp()
 {
 	QWhatsThis::showText( QCursor::pos(),
 				tr( "Here you can select your preferred "
@@ -1163,5 +1281,3 @@ void setupDialog::displayMIDIHelp( void )
 
 #include "moc_setup_dialog.cxx"
 
-
-#endif
